/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher.configuration;

import java.io.File;
import java.net.URL;
import java.util.Properties;

import org.ow2.petals.launcher.exception.InvalidDataRootPathException;
import org.ow2.petals.launcher.exception.UncompleteServerConfigurationException;
import org.ow2.petals.topology.generated.Topology;

/**
 * Instances of this interface give access to every information other classes
 * may need about the configuration of petals ESB.
 * 
 * @author Sebastien Andre - EBM WebSourcing
 */
public interface Configuration {

    /**
     * Name of the property defining the resource containing the topology
     * definition.
     */
    public static final String TOPOLOGY_URL_PROPERTY_NAME = "petals.topology.url";

    /**
     * Get the libraries directory
     * 
     * @return
     * @throws ConfigurationException
     */
    public File getLibDirectory() throws ConfigurationException;

    /**
     * Get the implementations directory
     * 
     * @return
     * @throws ConfigurationException
     */
    public File getImplementationsDirectory() throws ConfigurationException;

    /**
     * Get the extensions directory
     * 
     * @return The extension directory, or {@code null} if none is defined.
     * @throws ConfigurationException
     */
    public File getExtensionsDirectory() throws ConfigurationException;

    /**
     * Get the startup timeout in seconds
     * 
     * @return a positive integer
     * @throws ConfigurationException
     */
    public int getStartupTimeoutSeconds() throws ConfigurationException;

    /**
     * Get the topology of the Petals network in which this Petals node is
     * registered.
     * 
     * @return The topology of the Petals network
     * @throws UncompleteServerConfigurationException
     *             An error occurs reading the topology
     */
    public Topology getTopology() throws UncompleteServerConfigurationException;

    /**
     * <p>
     * Get the data root path of the Petals container.
     * </p>
     * <p>
     * If the data root path does not exist on the local filesystem, this method
     * is in charge of creating it.
     * </p>
     * 
     * @return The data root path of the Petals container
     * @throws InvalidDataRootPathException
     *             The root directory of Petals data is not a valid directory
     * @throws UncompleteServerConfigurationException
     *             An error occurs about the data root path
     */
    public File getDataRootPath() throws InvalidDataRootPathException,
            UncompleteServerConfigurationException;

    /**
     * Get the local container id
     * 
     * @return The local container identifier
     * @throws UncompleteServerConfigurationException
     *             The local container identifier is missing
     */
    public String getLocalContainerId() throws UncompleteServerConfigurationException;

    /**
     * Get the local properties of the current container
     */
    public Properties getServerLocalProperties();

    /**
     * @return The URL of the resource containing the local properties of the
     *         current container
     */
    public URL getServerPropertiesUrl();

}
