/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher.configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;
import org.ow2.petals.launcher.exception.InvalidDataRootPathException;
import org.ow2.petals.launcher.exception.InvalidServerPropertiesFileException;
import org.ow2.petals.launcher.exception.NoContainerConfigurationAvailableException;
import org.ow2.petals.launcher.exception.ServerPropertiesFileNotFoundException;
import org.ow2.petals.launcher.exception.UncompleteServerConfigurationException;
import org.ow2.petals.topology.TopologyBuilder;
import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.generated.Topology;

import com.ebmwebsourcing.easycommons.properties.PropertiesException;
import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;

/**
 * This class implements the default configuration for petals ESB.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class DefaultConfiguration implements Configuration {

    public static final String DEFAULT_TOPOLOGY_FILE = "topology.xml";

    public static final String DEFAULT_SERVER_PROPERTIES_FILE = "server.properties";

    public static final String PETALS_LIB_DIR_PROPERTY_NAME = "petals.lib.dir";

    public static final String PETALS_EXTENSIONS_DIR_PROPERTY_NAME = "petals.ext.dir";

    public static final String DEFAULT_DATA_ROOT_DIRECTORY = "data";
    
    public static final String PETALS_IMPLEMENTATIONS_DIRECTORY = "implementations";

    /**
     * The URL of the local configuration of the container. Not
     * <code>null</code>
     */
    private final URL serverPropertiesUrl;

    private final Properties serverLocalProperties;

    /**
     * Creates a new default configuration instance
     * 
     * @param serverPropertiesUrl
     *            URL to the resource containing the local properties of the container to launch. If <code>null</code>,
     *            the default server properties are read from the class-path resource
     *            {@value #DEFAULT_SERVER_PROPERTIES_FILE}
     * @throws UncompleteServerConfigurationException
     *             When the Petals server configuration is not complete
     * @throws InvalidServerPropertiesFileException
     *             When the Petals server configuration is invalid.
     * @throws NoContainerConfigurationAvailableException
     *             No configuration provided (<code>serverPropertiesUrl</code> is <code>null</code>) and no default
     *             configuration found on the classpath
     * @throws ServerPropertiesFileNotFoundException
     */
    public DefaultConfiguration(final URL serverPropertiesUrl)
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        this.serverLocalProperties = new Properties();

        InputStream isServerProperties = null;

        try {
            final String serverPropertiesUrlStr;
            if (serverPropertiesUrl != null) {
                serverPropertiesUrlStr = serverPropertiesUrl.toString();
                try {
                    isServerProperties = serverPropertiesUrl.openStream();
                    this.serverPropertiesUrl = serverPropertiesUrl;
                } catch (final FileNotFoundException e) {
                    throw new ServerPropertiesFileNotFoundException(serverPropertiesUrlStr, e);
                } catch (final IOException e) {
                    throw new InvalidServerPropertiesFileException(serverPropertiesUrlStr, e);
                }
            } else {
                final File serverPropertiesFile;
                final URL defaultServerPropertiesUrl = Thread.currentThread().getContextClassLoader()
                        .getResource(DEFAULT_SERVER_PROPERTIES_FILE);
                if (defaultServerPropertiesUrl != null) {
                    serverPropertiesUrlStr = defaultServerPropertiesUrl.toString();
                    try {
                        serverPropertiesFile = new File(defaultServerPropertiesUrl.toURI());
                        if (!serverPropertiesFile.exists()) {
                            throw new ServerPropertiesFileNotFoundException(serverPropertiesUrlStr);
                        } else if (!serverPropertiesFile.isFile()) {
                            throw new InvalidServerPropertiesFileException(
                                    serverPropertiesUrlStr);
                        } else {
                            try {
                                this.serverPropertiesUrl = serverPropertiesFile.toURI().toURL();
                                isServerProperties = new FileInputStream(serverPropertiesFile);
                            } catch (final FileNotFoundException e) {
                                throw new ServerPropertiesFileNotFoundException(serverPropertiesUrlStr, e);
                            } catch (final MalformedURLException e) {
                                throw new InvalidServerPropertiesFileException(serverPropertiesUrlStr, e);
                            }
                        }
                    } catch (final URISyntaxException e) {
                        throw new InvalidServerPropertiesFileException(serverPropertiesUrlStr, e);
                    }
                } else {
                    throw new NoContainerConfigurationAvailableException();
                }
            }

            if (isServerProperties != null) {
                try {
                    this.serverLocalProperties.load(isServerProperties);
                } catch (final IOException e) {
                    throw new UncompleteServerConfigurationException(e);
                }
            } else {
                throw new InvalidServerPropertiesFileException(serverPropertiesUrlStr);
            }

        } finally {
            try {
                if (isServerProperties != null) {
                    isServerProperties.close();
                }
            } catch (final IOException e) {
                // NOP: Exception discarded
            }
        }
    }

    @Override
    public File getLibDirectory() throws ConfigurationException {
        final String libDir = System.getProperty(PETALS_LIB_DIR_PROPERTY_NAME);
        if (libDir != null && !libDir.isEmpty()) {
            return new File(libDir);
        } else {
            throw new ConfigurationException(
                    String.format("The system property '%s' is empty or not defined.", PETALS_LIB_DIR_PROPERTY_NAME));
        }
    }

    @Override
    public File getImplementationsDirectory() throws ConfigurationException {
        return new File(getLibDirectory(), PETALS_IMPLEMENTATIONS_DIRECTORY);
    }

    @Override
    public File getExtensionsDirectory() throws ConfigurationException {
        final String extDir = System.getProperty(PETALS_EXTENSIONS_DIR_PROPERTY_NAME);
        if (extDir != null && !extDir.isEmpty()) {
            return new File(extDir);
        } else {
            return null;
        }
    }

    @Override
    public int getStartupTimeoutSeconds() throws ConfigurationException {
        return 20;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        // TODO
        return sb.toString();
    }

    @Override
    public Topology getTopology() throws UncompleteServerConfigurationException {

        try {
            final URL topologyUrl;
            final String topologyUrlStr = (String) this.serverLocalProperties
                    .get(TOPOLOGY_URL_PROPERTY_NAME);
            if (topologyUrlStr == null || topologyUrlStr.trim().isEmpty()) {
                // No topology URL is defined, we use a topology resource
                // relative to the local configuration of the container
                topologyUrl = new URL(this.serverPropertiesUrl, DEFAULT_TOPOLOGY_FILE);
            } else {
                topologyUrl = new URL(topologyUrlStr);
            }

            try (final InputStream isTopology = topologyUrl.openStream()) {
                return TopologyBuilder.createTopology(isTopology);
            } catch (final TopologyException e) {
                throw new UncompleteServerConfigurationException(e);
            }
        } catch (final MalformedURLException e) {
            throw new UncompleteServerConfigurationException(e);
        } catch (final IOException e) {
            throw new UncompleteServerConfigurationException(e);
        }
    }

    @Override
    public File getDataRootPath() throws InvalidDataRootPathException,
            UncompleteServerConfigurationException {

        try {
            final String dataRootPathPropertyValue = (String) this.serverLocalProperties
                    .get(ConfigurationProperties.DATA_ROOT_DIRECTORY_PROPERTY_NAME);
            final File dataRootPath;
            if (dataRootPathPropertyValue == null || dataRootPathPropertyValue.trim().isEmpty()) {
                // No data root path is defined. We use a path relative to the
                // local configuration of the container: ../data
                if (this.serverPropertiesUrl.getProtocol().equals("file")) {
                    dataRootPath = new File(new File(this.serverPropertiesUrl.toURI())
                            .getParentFile().getParentFile(), DEFAULT_DATA_ROOT_DIRECTORY);
                } else {
                    throw new UncompleteServerConfigurationException(
                            "The data root path is required if the local configuration (server.properties) is remote.");
                }
            } else {
                try {
                    dataRootPath = new File(PropertiesHelper.resolveString(
                            dataRootPathPropertyValue, this.serverLocalProperties));
                } catch (final PropertiesException e) {
                    throw new InvalidDataRootPathException(dataRootPathPropertyValue, e);
                }
            }

            if (dataRootPath.exists()) {
                if (!dataRootPath.isDirectory()) {
                    throw new InvalidDataRootPathException(dataRootPath.getAbsolutePath());
                }
            } else {
                if (!dataRootPath.mkdirs()) {
                    throw new InvalidDataRootPathException(dataRootPath.getAbsolutePath());
                }
            }

            return dataRootPath;

        } catch (final URISyntaxException e) {
            throw new UncompleteServerConfigurationException(e);
        }
    }

    @Override
    public String getLocalContainerId() throws UncompleteServerConfigurationException {
        final String id = (String) this.serverLocalProperties.get(ConfigurationProperties.CONTAINER_NAME);
        if (id == null || id.isEmpty()) {
            throw new UncompleteServerConfigurationException(
                    "Local container identifier is not defined.");
        }
        else {
            return id;
        }
    }

    @Override
    public Properties getServerLocalProperties() {
        return this.serverLocalProperties;
    }

    @Override
    public URL getServerPropertiesUrl() {
        return this.serverPropertiesUrl;
    }
}
