/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.launcher.configuration.Configuration;
import org.ow2.petals.launcher.configuration.ConfigurationException;
import org.ow2.petals.launcher.configuration.DefaultConfiguration;
import org.ow2.petals.launcher.exception.InvalidDataRootPathException;
import org.ow2.petals.launcher.exception.PetalsLauncherException;
import org.ow2.petals.launcher.exception.UncompleteServerConfigurationException;
import org.ow2.petals.topology.generated.Topology;

/**
 * @author Christophe DENEUX - Linagora
 */
public class PetalsClassLoaderTestCase {

    @Rule
    final public TemporaryFolder tempFolder = new TemporaryFolder();

    private static class MyConfiguration implements Configuration {

        private final File libDir;

        public MyConfiguration(final File libDir) {
            this.libDir = libDir;
        }

        @Override
        public File getLibDirectory() throws ConfigurationException {
            return this.libDir;
        }

        @Override
        public File getImplementationsDirectory() throws ConfigurationException {
            return new File(getLibDirectory(), DefaultConfiguration.PETALS_IMPLEMENTATIONS_DIRECTORY);
        }

        @Override
        public File getExtensionsDirectory() throws ConfigurationException {
            return null;
        }

        @Override
        public int getStartupTimeoutSeconds() throws ConfigurationException {
            return 0;
        }

        @Override
        public Topology getTopology() throws UncompleteServerConfigurationException {
            return null;
        }

        @Override
        public File getDataRootPath() throws InvalidDataRootPathException, UncompleteServerConfigurationException {
            return null;
        }

        @Override
        public String getLocalContainerId() throws UncompleteServerConfigurationException {
            return null;
        }

        @Override
        public Properties getServerLocalProperties() {
            return null;
        }

        @Override
        public URL getServerPropertiesUrl() {
            return null;
        }

    }

    private boolean isJarInClassloader(final String kernelJarName) throws IOException, PetalsLauncherException {

        // Set test env
        final File libDir = this.tempFolder.newFolder();
        final File kernelJar = new File(libDir, kernelJarName);
        FileUtils.touch(kernelJar);
        final File implemDir = new File(libDir, DefaultConfiguration.PETALS_IMPLEMENTATIONS_DIRECTORY);
        assertTrue(implemDir.mkdir());

        // Execute & Assertions
        try (final URLClassLoader cl = new PetalsContainerClassLoader(new MyConfiguration(libDir))) {
            return Arrays.asList(cl.getURLs()).contains(kernelJar.toURI().toURL());
        }
    }

    /**
     * Test the petals kernel jar regexp
     */
    @Test
    public final void testPetalsKernelRegexp() throws IOException, PetalsLauncherException {

        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1.2.3-SNAPSHOT.jar' not found",
                this.isJarInClassloader("petals-microkernel-1.2.3-SNAPSHOT.jar"));
        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1.2.3.jar' not found",
                this.isJarInClassloader("petals-microkernel-1.2.3.jar"));
        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1.2-SNAPSHOT.jar' not found",
                this.isJarInClassloader("petals-microkernel-1.2-SNAPSHOT.jar"));
        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1.2.jar' not found",
                this.isJarInClassloader("petals-microkernel-1.2.jar"));
        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1-SNAPSHOT.jar' not found",
                this.isJarInClassloader("petals-microkernel-1-SNAPSHOT.jar"));
        assertTrue("Petals Micro-Kernel JAR 'petals-microkernel-1.jar' not found",
                this.isJarInClassloader("petals-microkernel-1.jar"));

        try {
            this.isJarInClassloader("petals-launcher-1.2.3.jar");
            fail("PetalsLauncherException not thrown");
        } catch (final PetalsLauncherException e) {
            assertTrue("Unexpected PetalsLauncherException",
                    e.getMessage().contains("No file matching the microkernel filename pattern"));
        }
    }

}
