/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.contrib.java.lang.system.RestoreSystemProperties;
import org.junit.rules.TemporaryFolder;
import org.junit.rules.TestRule;
import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;
import org.ow2.petals.launcher.exception.InvalidDataRootPathException;
import org.ow2.petals.launcher.exception.InvalidServerPropertiesFileException;
import org.ow2.petals.launcher.exception.NoContainerConfigurationAvailableException;
import org.ow2.petals.launcher.exception.ServerPropertiesFileNotFoundException;
import org.ow2.petals.launcher.exception.UncompleteServerConfigurationException;
import org.ow2.petals.launcher.util.ClasspathUrlStreamHandler;
import org.ow2.petals.topology.TopologyBuilder;
import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.DomainMode;
import org.ow2.petals.topology.generated.JmxService;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of {@link DefaultConfiguration}
 * 
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class DefaultConfigurationTest {

    @Rule
    public final TestRule restoreSystemProperties = new RestoreSystemProperties();

    @Rule
    public TemporaryFolder testTemporaryFolder = new TemporaryFolder();

    /**
     * <p>
     * Check the creation of a {@link DefaultConfiguration} against error management where:
     * <ul>
     * <li>a local server configuration url set to <code>null</code>,</li>
     * <li>no default configuration is available on the classpath.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The exception {@link InvalidServerPropertiesFileException} is thrown.</li>
     * </ul>
     * </p>
     */
    @Test(expected = NoContainerConfigurationAvailableException.class)
    public final void testDefaultConfiguration_WithoutConfigurationProvidedOrDefaultConfigurationAvailable()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        new DefaultConfiguration(null);
    }

    /**
     * <p>
     * Check the creation of a {@link DefaultConfiguration} against error management where:
     * <ul>
     * <li>a local server configuration url set to <code>null</code>,</li>
     * <li>a default configuration is available on the classpath.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The configuration is read successfully.</li>
     * </ul>
     * </p>
     */
    @Test(expected = NoContainerConfigurationAvailableException.class)
    public final void testDefaultConfiguration_WithDefaultConfigurationAvailable()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals(DefaultConfiguration.DEFAULT_SERVER_PROPERTIES_FILE)) {
                    return super.getResources("default.server.properties");
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final Configuration conf = new DefaultConfiguration(null);
            final Properties serverProperties = conf.getServerLocalProperties();
            assertNotNull("Null server properties", serverProperties);
            assertEquals("Invalid container name found", "0",
                    serverProperties.getProperty(ConfigurationProperties.CONTAINER_NAME));
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the creation of a {@link DefaultConfiguration} where:
     * <ul>
     * <li>a local server configuration url is correctly set,</li>
     * <li>read permission on the local server is denied.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The error {@link ServerPropertiesFileNotFoundException} occurs</li>
     * </ul>
     * </p>
     */
    @Test(expected = ServerPropertiesFileNotFoundException.class)
    public final void testDefaultConfiguration_WithPermissionDenied()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException, IOException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);
        final File unreadbleServerPropsFile = this.testTemporaryFolder.newFile("server.properties");
        FileUtils.copyURLToFile(serverPropertiesUrl, unreadbleServerPropsFile);
        unreadbleServerPropsFile.setReadable(false);

        try {
            new DefaultConfiguration(unreadbleServerPropsFile.toURI().toURL());
        } finally {
            unreadbleServerPropsFile.setReadable(true);
        }
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getDataRootPath()} where:
     * <ul>
     * <li>a local server configuration url is correctly set but using a not filed-based URL,</li>
     * <li>no data root path is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results: The error {@link UncompleteServerConfigurationException} occurs.
     * </p>
     */
    @Test(expected = UncompleteServerConfigurationException.class)
    public final void testGetDataRootPath_WithDefaultDataRootPathAsDirectoryThroughNotFileBasedURL()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            InvalidDataRootPathException, URISyntaxException, MalformedURLException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "classpath:default.server.properties";
        final URL serverPropertiesUrl = new URL(null, serverPropertiesName, new ClasspathUrlStreamHandler());
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getDataRootPath();
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getDataRootPath()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>no data root path is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The data root path is the provided one.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testGetDataRootPath_WithDefaultDataRootPathAsDirectoryThroughFileBasedURL()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            InvalidDataRootPathException, URISyntaxException, MalformedURLException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "data-root.exist.is-dir/conf/server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final File expectedDataRootPath = new File(
                new File(serverPropertiesUrl.toURI()).getParentFile().getParentFile(),
                DefaultConfiguration.DEFAULT_DATA_ROOT_DIRECTORY);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        final File dataRootPath = conf.getDataRootPath();
        assertNotNull("Null data root path returned", dataRootPath);
        assertEquals("Unexpected data root path", expectedDataRootPath, dataRootPath);
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getDataRootPath()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>an empty data root path is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The data root path is the default one, relative to the parent directory of the local server configuration.
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public final void testGetDataRootPath_WithEmptyDataRootPathAsDirectoryThroughFileBasedURL()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            InvalidDataRootPathException, URISyntaxException, MalformedURLException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "data-root.exist.is-dir/conf/server-empty-data-root.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final File expectedDataRootPath = new File(
                new File(serverPropertiesUrl.toURI()).getParentFile().getParentFile(),
                DefaultConfiguration.DEFAULT_DATA_ROOT_DIRECTORY);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        final File dataRootPath = conf.getDataRootPath();
        assertNotNull("Null data root path returned", dataRootPath);
        assertEquals("Unexpected data root path", expectedDataRootPath, dataRootPath);
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getDataRootPath()} where:
     * <ul>
     * <li>a local server configuration url is correctly set,</li>
     * <li>a data root path is set in the local server configuration,</li>
     * <li>the data root path is a directory.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The data root path is the provided one.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testGetDataRootPath_WithSetDataRootPathAsDirectory()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            InvalidDataRootPathException, NoContainerConfigurationAvailableException,
            ServerPropertiesFileNotFoundException, IOException {

        final String serverPropertiesName = "data-root.exist.is-dir/conf/server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);
        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);

        // We build the data root path directory
        final File expectedDataRootPath = testTemporaryFolder.newFolder("data-root");
        // We override the value of the property associated to the data root
        // path
        final Properties serverProperties = conf.getServerLocalProperties();
        serverProperties.setProperty(ConfigurationProperties.DATA_ROOT_DIRECTORY_PROPERTY_NAME,
                expectedDataRootPath.getAbsolutePath());

        final File dataRootPath = conf.getDataRootPath();
        assertNotNull("Null data root path returned", dataRootPath);
        assertEquals("Unexpected data root path", expectedDataRootPath, dataRootPath);
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getDataRootPath()} where:
     * <ul>
     * <li>a local server configuration url is correctly set,</li>
     * <li>a data root path is set in the local server configuration,</li>
     * <li>the data root path is a file.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The error {@link InvalidDataRootPathException} occurs.</li>
     * </ul>
     * </p>
     */
    @Test(expected = InvalidDataRootPathException.class)
    public final void testGetDataRootPath_WithDefaultDataRootPathAsFile() throws UncompleteServerConfigurationException,
            InvalidServerPropertiesFileException, InvalidDataRootPathException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "data-root.exist.is-file/conf/server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);
        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getDataRootPath();
    }

    /**
     * <p>
     * Check the creation of a {@link DefaultConfiguration} where:
     * <ul>
     * <li>no local server configuration url is set when creating the configuration</li>
     * <li>no default local server configuration is available on the class-path</li>
     * </p>
     * <p>
     * Expected results: An error occurs because no default local server configuration is available on the class-path
     * </p>
     */
    @Test(expected = NoContainerConfigurationAvailableException.class)
    public final void testDefaultConfiguration_Default_WithoutDefaultResource()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException,
            InvalidDataRootPathException, URISyntaxException {

        // Set the test environment
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(DefaultConfiguration.DEFAULT_SERVER_PROPERTIES_FILE);
        assertNull("Default resource 'server.properties' found", serverPropertiesUrl);

        // Execute
        new DefaultConfiguration(null);
    }

    /**
     * <p>
     * Check the creation of a {@link DefaultConfiguration} where:
     * <ul>
     * <li>no local server configuration url is set when creating the configuration</li>
     * <li>a default local server configuration is available on the class-path</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The configuration is read successfully with all default values.</li>
     * </ul>
     * </p>
     */
    @Test
    public final void testDefaultConfiguration_Default_WithDefaultResource()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException,
            InvalidDataRootPathException, URISyntaxException {

        // Set the test environment
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassLoader) {

            @Override
            public URL getResource(final String name) {
                if (DefaultConfiguration.DEFAULT_SERVER_PROPERTIES_FILE.equals(name)) {
                    return serverPropertiesUrl;
                } else {
                    return super.getResource(name);
                }
            }
        });

        // Execute
        try {
            final Configuration conf = new DefaultConfiguration(null);

            // Assertions
            final Properties serverProperties = conf.getServerLocalProperties();
            assertNotNull("Null server properties", serverProperties);
            assertEquals("Invalid container name found", "0",
                    serverProperties.getProperty(ConfigurationProperties.CONTAINER_NAME));
            assertEquals("Invalid container name found", "0", conf.getLocalContainerId());

            final File expectedDataRootPath = new File(
                    new File(serverPropertiesUrl.toURI()).getParentFile().getParentFile(),
                    DefaultConfiguration.DEFAULT_DATA_ROOT_DIRECTORY);
            assertEquals(expectedDataRootPath, conf.getDataRootPath());

            final Topology readTopology = conf.getTopology();
            assertNotNull(readTopology);
            assertEquals("PEtALS-default-topology", readTopology.getDomain().getName());

        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLocalContainerId()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>no container name is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results: An error occurs because the container name is not set
     * </p>
     */
    @Test(expected = UncompleteServerConfigurationException.class)
    public final void testGetLocalContainerId_NoSet()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "no-container-name.server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getLocalContainerId();
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLocalContainerId()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>an empty container name is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results: An error occurs because the container name is not set
     * </p>
     */
    @Test(expected = UncompleteServerConfigurationException.class)
    public void testGetLocalContainerId_Empty()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "empty-container-name.server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getLocalContainerId();
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLocalContainerId()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>no topology URL is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The topology is read from the default topology URL.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetTopology_NoSet()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "no-topology.server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);

        final Topology readTopology = conf.getTopology();
        assertNotNull(readTopology);
        assertEquals("PEtALS-default-topology", readTopology.getDomain().getName());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLocalContainerId()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>an empty topology URL is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The topology is read from the default topology URL.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetTopology_Empty()
            throws UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final String serverPropertiesName = "empty-topology.server.properties";
        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource(serverPropertiesName);
        assertNotNull("Resource '" + serverPropertiesName + "' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);

        final Topology readTopology = conf.getTopology();
        assertNotNull(readTopology);
        assertEquals("PEtALS-default-topology", readTopology.getDomain().getName());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLocalContainerId()} where:
     * <ul>
     * <li>a local server configuration url is correctly set using a filed-based URL,</li>
     * <li>a topology URL is set in the local server configuration.</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>The topology is correctly read from the URL set in the local server configurationL.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetTopology_SetAsReadableFile()
            throws IOException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException, TopologyException {

        // Set the test env
        final File confRoot = this.testTemporaryFolder.newFolder();
        final File serverPropertiesFile = File.createTempFile("srv", "properties", confRoot);
        final File topologyFile = File.createTempFile("topology", "xml", confRoot);

        final Topology topology = new Topology();
        final Domain domain = new Domain();
        final String expectedDomainName = "my-expected-domain";
        domain.setName(expectedDomainName);
        domain.setMode(DomainMode.STATIC);
        topology.setDomain(domain);
        final Container container = new Container();
        container.setName("my-container");
        container.setHost("localhost");
        final JmxService jmxService = new JmxService();
        jmxService.setRmiPort(7700);
        container.setJmxService(jmxService);
        domain.getContainer().add(container);
        TopologyBuilder.marshallTopologyFile(topologyFile, topology);

        final String tplServerPropertiesName = "default.server.properties";
        final InputStream serverPropertiesStream = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream(tplServerPropertiesName);
        assertNotNull("Resource '" + tplServerPropertiesName + "' not found", serverPropertiesStream);
        final Properties serverProperties = new Properties();
        serverProperties.load(serverPropertiesStream);
        serverProperties.setProperty(DefaultConfiguration.TOPOLOGY_URL_PROPERTY_NAME,
                topologyFile.toURI().toURL().toString());
        try (final OutputStream osServerProperties = new FileOutputStream(serverPropertiesFile)) {
            serverProperties.store(osServerProperties, null);
        }

        // Execute
        final URL serverPropertiesUrl = serverPropertiesFile.toURI().toURL();
        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);

        final Topology readTopology = conf.getTopology();
        assertNotNull(readTopology);
        assertEquals(expectedDomainName, readTopology.getDomain().getName());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLibDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_LIB_DIR_PROPERTY_NAME} is not set</li>
     * </p>
     * <p>
     * Expected results: an error occurs about the system property
     * {@value DefaultConfiguration#PETALS_LIB_DIR_PROPERTY_NAME} not set.
     * </p>
     */
    @Test(expected = ConfigurationException.class)
    public void testGetLibDir_NoSet()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getLibDirectory();
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLibDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_LIB_DIR_PROPERTY_NAME} is empty</li>
     * </p>
     * <p>
     * Expected results: an error occurs about the empty system property
     * {@value DefaultConfiguration#PETALS_LIB_DIR_PROPERTY_NAME}.
     * </p>
     */
    @Test(expected = ConfigurationException.class)
    public void testGetLibDir_Empty()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);
        System.setProperty(DefaultConfiguration.PETALS_LIB_DIR_PROPERTY_NAME, "");

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        conf.getLibDirectory();
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getLibDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_LIB_DIR_PROPERTY_NAME} is correctly set</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The expected value for lib dir is returned,</li>
     * <li>The expected value for mutable implementation dir is returned,</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetLibDir_Set()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException, IOException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);
        final File expectedLibDir = this.testTemporaryFolder.newFolder("my-expected-lib-dir");
        System.setProperty(DefaultConfiguration.PETALS_LIB_DIR_PROPERTY_NAME, expectedLibDir.getAbsolutePath());

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        assertEquals(expectedLibDir, conf.getLibDirectory());
        assertEquals(new File(expectedLibDir, DefaultConfiguration.PETALS_IMPLEMENTATIONS_DIRECTORY),
                conf.getImplementationsDirectory());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getExtensionsDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_EXTENSIONS_DIR_PROPERTY_NAME} is not set</li>
     * </p>
     * <p>
     * Expected results: {@code null} is returned because the system property
     * {@value DefaultConfiguration#PETALS_EXTENSIONS_DIR_PROPERTY_NAME} not set.
     * </p>
     */
    @Test
    public void testGetExtensionsDirectory_NoSet()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        assertNull(conf.getExtensionsDirectory());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getExtensionsDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_EXTENSIONS_DIR_PROPERTY_NAME} is empty</li>
     * </p>
     * <p>
     * Expected results: {@code null} is returned because the system property
     * {@value DefaultConfiguration#PETALS_EXTENSIONS_DIR_PROPERTY_NAME} is empty.
     * </p>
     */
    @Test
    public void testGetExtensionsDirectory_Empty()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);

        System.setProperty(DefaultConfiguration.PETALS_EXTENSIONS_DIR_PROPERTY_NAME, "");

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        assertNull(conf.getExtensionsDirectory());
    }

    /**
     * <p>
     * Check the method {@link DefaultConfiguration#getExtensionsDirectory()} when:
     * <ul>
     * <li>the system property {@value DefaultConfiguration#PETALS_EXTENSIONS_DIR_PROPERTY_NAME} is correctly set</li>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>No error occurs,</li>
     * <li>The expected value for ext dir is returned.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetExtensionsDirectory_Set()
            throws ConfigurationException, UncompleteServerConfigurationException, InvalidServerPropertiesFileException,
            NoContainerConfigurationAvailableException, ServerPropertiesFileNotFoundException, IOException {

        final URL serverPropertiesUrl = Thread.currentThread().getContextClassLoader()
                .getResource("default.server.properties");
        assertNotNull("Resource 'default.server.properties' not found", serverPropertiesUrl);

        final File expectedExtDir = this.testTemporaryFolder.newFolder("my-expected-ext-dir");
        System.setProperty(DefaultConfiguration.PETALS_EXTENSIONS_DIR_PROPERTY_NAME, expectedExtDir.getAbsolutePath());

        final Configuration conf = new DefaultConfiguration(serverPropertiesUrl);
        assertEquals(expectedExtDir, conf.getExtensionsDirectory());
    }

}
